==> sessiond-0.3.tar.gz: sessiond-0.3/data.cpp <==
// sessiond - SSL session cache daemon, file data.cpp
// Copyright (C) 2009 Michal Trojnara <Michal.Trojnara@mirt.net>
//
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the
// Free Software Foundation; either version 2 of the License, or (at your
// option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, see <http://www.gnu.org/licenses>.
//
// Linking sessiond statically or dynamically with other modules is making
// a combined work based on sessiond. Thus, the terms and conditions of
// the GNU General Public License cover the whole combination.

#include "data.h"
#include <stdio.h>

void DATA::insert(const BYTES &k, const BYTES &v, const unsigned int timeout) {
    time_t t=time(NULL);
    cleanup(t); // purge expired entries
    storage[k].v=v;
    storage[k].t=t+timeout;
    log[t+timeout].insert(v);
}

const BYTES DATA::find(const BYTES &k) {
    return storage[k].v;
}

const bool DATA::exists(const BYTES &k) {
    bool retval=storage.count(k)!=0;
    if(retval)
        ++stats_hits;
    else
        ++stats_misses;
    return retval;
}

void DATA::erase(const BYTES &k) {
    time_t t=storage[k].t;
    log[t].erase(k);
    if(log[t].empty())
        log.erase(t);
    storage.erase(k);
}

void DATA::cleanup(const time_t t) {
    typedef map<time_t, set<BYTES> >::iterator log_iterator;
    typedef set<BYTES>::iterator set_iterator;
    log_iterator begin=log.begin(), end=log.lower_bound(t);
    for(log_iterator i=begin; i!=end; ++i) // remove expired data entries
        for(set_iterator j=i->second.begin(); j!=i->second.end(); ++j)
            storage.erase(*j);
    log.erase(begin, end); // remove old log entries
    while(storage.size()>1000000) { // DoS protection (cache size limit)
        log_iterator i=log.begin(); // next to expire
        for(set_iterator j=i->second.begin(); j!=i->second.end(); ++j)
            storage.erase(*j); // erase all entries created within 1 second
        log.erase(i);
    }
}

char *DATA::stats() {
    unsigned int stats_total=stats_hits+stats_misses;
    snprintf(stats_txt, sizeof stats_txt,
        "cache size=%u, hits=%u, misses=%u, ratio=%2.2f%%",
        storage.size(), stats_hits, stats_misses,
        stats_total>0 ? 100.0*stats_hits/stats_total : 0.0);
    return stats_txt;
}

// end of data.cpp

==> sessiond-0.4.tar.gz: sessiond-0.4/data.cpp <==
// sessiond - SSL session cache daemon, file data.cpp
// Copyright (C) 2009 Michal Trojnara <Michal.Trojnara@mirt.net>
//
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the
// Free Software Foundation; either version 2 of the License, or (at your
// option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, see <http://www.gnu.org/licenses>.
//
// Linking sessiond statically or dynamically with other modules is making
// a combined work based on sessiond. Thus, the terms and conditions of
// the GNU General Public License cover the whole combination.

#include "data.h"

void DATA::insert(const BYTES &k, const BYTES &v, const unsigned int timeout) {
    time_t t=time(NULL);
    cleanup(t); // purge expired entries
    storage[k].v=v;
    storage[k].t=t+timeout;
    log[t+timeout].insert(v);
}

const BYTES DATA::find(const BYTES &k) {
    return storage[k].v;
}

const bool DATA::exists(const BYTES &k) {
    return storage.count(k)!=0;
}

void DATA::erase(const BYTES &k) {
    time_t t=storage[k].t;
    log[t].erase(k);
    if(log[t].empty())
        log.erase(t);
    storage.erase(k);
}

void DATA::cleanup(const time_t t) {
    typedef map<time_t, set<BYTES> >::iterator log_iterator;
    typedef set<BYTES>::iterator set_iterator;
    log_iterator begin=log.begin(), end=log.lower_bound(t);
    for(log_iterator i=begin; i!=end; ++i) // remove expired data entries
        for(set_iterator j=i->second.begin(); j!=i->second.end(); ++j)
            storage.erase(*j);
    log.erase(begin, end); // remove old log entries
    while(storage.size()>1000000) { // DoS protection (cache size limit)
        log_iterator i=log.begin(); // next to expire
        for(set_iterator j=i->second.begin(); j!=i->second.end(); ++j)
            storage.erase(*j); // erase all entries created within 1 second
        log.erase(i);
    }
}

// end of data.cpp
